
import requests
from . import APIUrls

def getState():
    """
    Shows informations about api.
    """

    headers = {'accept': 'application/json'}
    stateData = requests.get(APIUrls.lnapi+APIUrls.stateUrl,headers=headers)

    if stateData.status_code == 200:
        stateData = stateData.json()
        if stateData['state']['newPosition']:
            return stateData
        else:
            raise ValueError('New Positions currently not allowed')
    else:
        raise RuntimeError('Unable to fetch State Information: %s' % stateData.text)


def getNodeInformation():
    """
    Shows informations about the lightning node.
    """
    
    headers = {'accept': 'application/json'}
    nodeData = requests.get(APIUrls.lnapi+APIUrls.nodeUrl,headers=headers)

    if nodeData.status_code == 200:
        return nodeData.json()
    else:
        raise RuntimeError('Unable to fetch Node Information: %s' % nodeData.text)




import requests
from . import APIUrls

def getIndex():
    """
    Retrieves index value.
    """

    headers = {'accept': 'application/json'}
    indexData = requests.get(APIUrls.lnapi+APIUrls.indexUrl,headers=headers)

    if indexData.status_code == 200:
        return indexData.json()
    else:
        raise RuntimeError('Unable to fetch index data: %s'%indexData.text)

def getBidOffer():
    """
    Retrieves bid-offer values.
    """

    headers = {'accept': 'application/json'}
    bidOfferData = requests.get(APIUrls.lnapi+APIUrls.bidOfferUrl,headers=headers)

    if bidOfferData.status_code == 200:
        return bidOfferData.json()
    else:
        raise RuntimeError('Unable to fetch bid-offer data: %s'%bidOfferData.text)
    




import requests
from . import APIUrls

def userInformation(token):    
    """
    Get the user account Information.

    Parameters-
    token: Authentication token.
    """
    headers = {
    'accept': "application/json",
    'authorization': "Bearer %s" % token
    }
    userInfo = requests.get(APIUrls.lnapi+APIUrls.userUrl,headers=headers)
    if userInfo.status_code == 200:
        return userInfo.json()
    else:
        raise RuntimeError('Unable to get user information: %s' % userInfo.text)

def getBalance(token):
    """
    Get the user account balance.

    Parameters-
    token: Authentication token.
    """
    
    return userInformation(token)['balance']

def updateUser(token, leaderboard = None, showUsername = None, username = None):    
    """
    Update user account information.

    Parameters-
    token: Authentication token.
    leaderboard: True, if allowed to show user's profit on leaderboard.
    showUsername: True, if allowed to show the username on LN Marktes public data.
    username: username to display.
    """

    headers = {
    'content-type': "application/json",        
    'accept': "application/json",
    'authorization': "Bearer %s" % token
    }
    payload = "{"
    comma = False
    if showUsername != None:
        payload += "\"show_username\":%s" % str(showUsername).lower()
        comma = True
    if leaderboard != None:
        if comma:
            payload += ",\"show_leaderboard\":%s" % str(leaderboard).lower()
        else:
            payload += "\"show_leaderboard\":%s" % str(leaderboard).lower()
        comma = True
    if username == None:
        payload += "}"
    else:
        if comma:
            payload += ",\"username\":\"%s\"}" % username
        else:
            payload += "\"username\":\"%s\"}" % username

    userInfo = requests.put(APIUrls.lnapi+APIUrls.userUrl,data=payload,headers=headers)
    if userInfo.status_code == 200:
        return userInfo.json()
    else:
        raise RuntimeError('Unable to update user information: %s' % userInfo.text)


def updatePassword(token, previousPassword, newPassword):
    """
    Update user account information.

    Parameters-
    token: Authentication token.
    previousPassword: Previous Password.
    newPassword: New Password.
    """
    headers = {
    'content-type': "application/json",        
    'authorization': "Bearer %s" % token
    }
    payload="{\"previousPassword\":\"%s\",\"newPassword\":\"%s\"}" %(previousPassword,newPassword)

    updateInfo = requests.put(APIUrls.lnapi+APIUrls.updateUrl,data=payload,headers=headers)
    return updateInfo.ok

def getTransactions(token, type, nbitem=-1, index=1, getLength=False, start=None, end=None):    
    """
    Retrieves all withdraw or deposit you did with LN Markets or just one page

    Parameters-
    token: Authentication token.
    type: what kind of transactions ('withdraw' or 'deposit') to fetch
    nbitem: maximum length of returned table (optional)
    index: page offset (optional)
    getLength: Boolean. True if need length of date splice (optional)
    start: start of time interval (optional)
    end: end of time interval (optional)
    """

    headers = {
    'accept': "application/json",
    'authorization': "Bearer %s" % token
    }
    params = { "type": type, "getLength": str(getLength).lower()}
    if nbitem != -1:
        params['nbitem'] = nbitem
        params['index'] = index
    if start != None:
        params['start'] = start
    if end != None:
        params['end'] = end
        
    transactions = requests.get(APIUrls.lnapi+APIUrls.userHistoryUrl,headers=headers,params=params)
    if transactions.status_code == 200:
        return transactions.json()
    else:
        raise RuntimeError('Unable to get user transactions: %s' % transactions.text)

def getTokens():
    """
    Retrieves the list of active JSON Web Token user currently holds.
    """
    
    headers = {
    'accept': "application/json"
    }
        
    tokens = requests.get(APIUrls.lnapi+APIUrls.tokenUrl,headers=headers)
    if tokens.status_code == 200:
        return tokens.json()
    else:
        raise RuntimeError('Unable to get user tokens: %s' % tokens.text)

def revokeToken(token=None):
    """
    identifies and revoke usage of a given JWT.

    Parameters-
    token: Identifier of Token to be removed (optional)
    """
    
    if token == None:   
        tokenInfo = requests.delete(APIUrls.lnapi+tokenUrl)
    else:
        params = {"jti": token}
        tokenInfo = requests.delete(APIUrls.lnapi+APIUrls.tokenUrl,params=params)        
    if tokenInfo.status_code == 200:
        return True
    else:
        raise RuntimeError('Unable to revoke user token: %s' % tokenInfo.text)

def generateToken(expiry,deposit=False,withdraw=False,positions=False,user=False):
    """
    Using the given scopes, generate token to give access to different parts of the public API.

    Parameters-
    expiry: token expiry in seconds.
    deposit: Allow deposit (optional)
    withdraw: Allow withdraw (optional)
    positions: Allow opening or closing trade positions (optional)
    user: Allow access to user information (optional)
    """

    headers = {
        'content-type': "application/json",
        'accept': "application/json"
    }

    payload = "{\"expiry\":%d,\"scopes\":[" % expiry
    putComma = False
    if deposit:
        if putComma:
            payload+=","
        payload+="\"deposit\""
        putComma=True
    if withdraw:
        if putComma:
            payload+=","
        payload+="\"withdraw\""
        putComma=True
    if positions:
        if putComma:
            payload+=","
        payload+="\"positions\""
        putComma=True
    if user:
        if putComma:
            payload+=","
        payload+="\"user\""
        putComma=True
    payload+="]}"
    
    tokenInfo = requests.post(APIUrls.lnapi+APIUrls.tokenUrl,data=payload,headers=headers)        
    if tokenInfo.status_code == 200:
        return tokenInfo.json()['token']
    else:
        raise RuntimeError('Unable to generate user token: %s' % tokenInfo.text)

def deposit(token, amount, unit="sat"):
    """
    Add fund to your LN Markets balance.

    Parameters-
    token: Authentication token
    amount: amount to deposit
    unit: sat by default (optional)

    Returns-
    paymentRequest: Invoice to pay to add balanceto the account
    expiry: time in seconds before expiry of request
    """
    
    headers = {
        'content-type': "application/json",
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    payload = "{\"amount\":%d,\"unit\":\"%s\"}" % (amount, unit)
    
    invoice = requests.post(APIUrls.lnapi+APIUrls.depositUrl,data=payload,headers=headers)
    if invoice.status_code == 200:
        return invoice.json()
    else:
        raise RuntimeError('Unable to get invoice: %s' % invoice.text)

def withdrawInvoice(token, amount, invoice, unit="sat"):
    """
    Move funds from the Lightning channel with LN Markets to user wallet by using an invoice directly.

    Parameters-
    token: Authentication token
    amount: amount to withdraw
    unit: Currently only sat (optional)
    invoice: BOLT 11 invoice with same amount

    Returns-
    wid: withdraw ID
    amount: amount withdrawn
    paymentSecret: Payment secret of settled invoice
    paymentHash: Payment hash of settled invoice
    """
    
    headers = {
        'content-type': "application/json",
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    payload = "{\"amount\":%d,\"unit\":\"%s\",\"invoice\":\"%s\"}" % (amount,unit,invoice)
    
    payment = requests.post(APIUrls.lnapi+APIUrls.withdrawUrl,data=payload,headers=headers)
    if payment.status_code == 200:
        return payment.json()
    else:
        raise RuntimeError('Unable to make payment: %s' % payment.text)

def withdrawLNURL(token, amount, unit="sat"):
    """
    Move funds from the Lightning channel with LN Markets to user wallet by using an LNURL.

    Parameters-
    token: Authentication token
    amount: amount to withdraw
    unit: Currently only sat (optional)

    Returns-
    lnurl: LNURL BECH32 encoded string for wallet
    """
    
    headers = {
        'content-type': "application/json",
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    payload = "{\"amount\":%d,\"unit\":\"%s\"}" % (amount,unit)
    
    LNUrlInfo = requests.post(APIUrls.lnapi+APIUrls.withdrawLNUrl,data=payload,headers=headers)
    if LNUrlInfo.status_code == 200:
        return LNUrlInfo.json()
    else:
        raise RuntimeError('Unable to create LNURL: %s' % LNUrlInfo.text)
    
def login(username, password):
    """
    Use existing credentials to log in.
    """
    
    headers = {
        'content-type': "application/json",
        'accept': "application/json"
    }
    payload = "{\"login\":\"%s\",\"password\":\"%s\"}" % (username,password)
    loginResp = requests.post(APIUrls.lnapi+APIUrls.loginUrl,data=payload,headers=headers)
    if loginResp.status_code == 200:
        return loginResp.json()
    else:
        raise RuntimeError('Unable to login: %s' % loginResp.text)

def logout():
    """
    Deletes session cookie
    """

    return requests.post(APIUrls.lnapi+APIUrls.logoutUrl).ok

    



from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
    
setup(
    name='LNMarkets',
    packages=find_packages(include=['LNMarkets']),
    version='0.1.1',
    description='LNMarkets API Wrapper',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DivyanshuBagga/LNMarkets",
    author='Divyanshu Bagga',
    author_email="divyanshu.baggar@pm.me",
    license='MIT',
    install_requires=['requests'],
    setup_requires=['pytest-runner'],
    tests_require=['pytest==4.4.1'],
    test_suite='tests',
    download_url = 'https://github.com/DivyanshuBagga/LNMarkets/archive/0.1.1.tar.gz',
    keywords = ['Bitcoin','Finance','Trading'],
)




import requests
from . import APIUrls

def getPositions(token, type = None):
    """
    Retrieves either opened, closed or all positions.

    Parameters-
    token: Authentication token.
    type: what kind of positions ('open' or 'closed' or 'all') to fetch (optional)

    """
    
    headers = {
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    if type == None:
        positionData = requests.get(APIUrls.lnapi+APIUrls.positionUrl,headers=headers)
        if positionData.status_code == 200:
            return positionData.json()
        else:
            raise RuntimeError('Unable to get positions')
    else:
        params = {"type":type}
        positionData = requests.get(APIUrls.lnapi+APIUrls.positionUrl,params = params,headers=headers)
        if positionData.status_code == 200:
            return positionData.json()
        else:
            raise RuntimeError('Unable to get positions: %s' % positionData.text)


def createPosition(token, type, side, leverage, margin=None, quantity=None, stoploss=None, takeprofit=None, price=None):
    """
    Send the order form parameters to add a new position in database. If type="l", the property price must be included in the request to know when the position should be filled. You can choose to use the margin or the quantity as a parameter, the other will be calculated with the one you choosed.

    Parameters-
    token: Authentication token.
    type: "l" for limit order, "m" for market order.
    side: "b" for buy, "s" for sell.
    leverage: Leverage of the order.
    margin: margin or quantity must be given (optional)
    quantity: margin or quantity must be given (optional)
    stoploss: StopLoss level. (optional)
    takeprofit: Profit taking level. (optional)
    price: limit price for limit order. (optional)
    """
    if margin == None and quantity == None:
        raise ValueError('Either margin or quantity must be provided')
    if type == "l" and price == None:
        raise ValueError('Limit Price must be pprovided for limit order')

    payload = "{\"type\":\"%s\",\"side\":\"%s\",\"leverage\":%.2f" %(type, side, leverage)

    if type == "l":
        payload += ",\"price\":%.2f" % price
    if margin == None:
        payload += ",\"quantity\":%d" % quantity
    else:
        payload += ",\"margin\":%d" % margin
    if stoploss != None:
        payload += ",\"stoploss\":%.2f" % stoploss
    if takeprofit != None:
        payload += ",\"takeprofit\":%.2f" % takeprofit

    payload += "}"        
        
    headers = {
        'content-type': "application/json",
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    positionData = requests.post(APIUrls.lnapi+APIUrls.positionUrl,data=payload,headers=headers)
    if positionData.status_code == 200:
        return positionData.json()
    else:
        raise RuntimeError('Unable to create position: %s' % positionData.text)

def buy(token, leverage, margin=None, quantity=None, stoploss=None, takeprofit=None):
    """
    Send the buy order. You can choose to use the margin or the quantity as a parameter, the other will be calculated with the one you choosed.

    Parameters-
    token: Authentication token.
    leverage: Leverage of the order.
    margin: margin or quantity must be given (optional)
    quantity: margin or quantity must be given (optional)
    stoploss: StopLoss level. (optional)
    takeprofit: Profit taking level. (optional)
    """
    return createPosition(token,"m","b",leverage,margin,quantity,stoploss,takeprofit)

def sell(token, leverage, margin=None, quantity=None, stoploss=None, takeprofit=None):
    """
    Send the sell order. You can choose to use the margin or the quantity as a parameter, the other will be calculated with the one you choosed.

    Parameters-
    token: Authentication token.
    leverage: Leverage of the order.
    margin: margin or quantity must be given (optional)
    quantity: margin or quantity must be given (optional)
    stoploss: StopLoss level. (optional)
    takeprofit: Profit taking level. (optional)
    """
    return createPosition(token,"m","s",leverage,margin,quantity,stoploss,takeprofit)

def limitBuy(token, leverage, price, margin=None, quantity=None, stoploss=None, takeprofit=None):
    """
    Send the limit buy order. You can choose to use the margin or the quantity as a parameter, the other will be calculated with the one you choosed.

    Parameters-
    token: Authentication token.
    leverage: Leverage of the order.
    price: Limit price of order.
    margin: margin or quantity must be given (optional)
    quantity: margin or quantity must be given (optional)
    stoploss: StopLoss level. (optional)
    takeprofit: Profit taking level. (optional)
    """
    return createPosition(token,"l","b",leverage,margin,quantity,stoploss,takeprofit,price)

def limitSell(token, leverage, price, margin=None, quantity=None, stoploss=None, takeprofit=None):
    """
    Send the limit sell order. You can choose to use the margin or the quantity as a parameter, the other will be calculated with the one you choosed.

    Parameters-
    token: Authentication token.
    leverage: Leverage of the order.
    price: Limit price of order.
    margin: margin or quantity must be given (optional)
    quantity: margin or quantity must be given (optional)
    stoploss: StopLoss level. (optional)
    takeprofit: Profit taking level. (optional)
    """
    return createPosition(token,"l","s",leverage,margin,quantity,stoploss,takeprofit,price)

def updatePosition(token, pid, type, value):
    """
    Allows user to modify stoploss or takeprofit parameters of an existing position.

    Parameters-
    token: Authentication token.
    pid: ID of the position.
    type: "takeprofit" or "stoploss".
    value: Price level to set.
    """

    payload = "{\"pid\":\"%s\",\"type\":\"%s\",\"value\":%.2f}" %(pid, type, value)
    headers = {
        'content-type': "application/json",
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    positionData = requests.put(APIUrls.lnapi+APIUrls.positionUrl,data=payload,headers=headers)
    if positionData.status_code == 200:
        return positionData.json()
    else:
        raise RuntimeError('Unable to update position %s, reason: %s' % (pid,positionData.text))

def closePosition(token, pid):
    """
    Close the user position, the PL will be calculated against the current bid or offer depending on the side of the position.
 
    Parameters-
    token: Authentication token.
    pid: ID of the position.
    """

    params = {"pid": pid}
    headers = {
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    positionData = requests.delete(APIUrls.lnapi+APIUrls.positionUrl,params=params,headers=headers)
    if positionData.status_code == 200:
        return positionData.json()
    else:
        raise RuntimeError('Unable to close position %s, reason: %s' % (pid,positionData.text))
        
def closeAllLongs(token):
    """
    Close all positions on long side.
 
    Parameters-
    token: Authentication token.
    """   

    positionData = getPositions(token, "open")['open']
    pl = 0.0
    for position in positionData:
        if position['side'] == "b" and not position['closed'] and not position['canceled']:
            closeData = closePosition(token, position['pid'])
            pl += float(closeData['pl'])

    return pl;

def closeAllShorts(token):
    """
    Close all positions on short side.
 
    Parameters-
    token: Authentication token.
    """   
    
    positionData = getPositions(token, "open")['open']
    pl = 0.0
    for position in positionData:
        if position['side'] == "s" and not position['closed'] and not position['canceled']:
            closeData = closePosition(token, position['pid'])
            pl += float(closeData['pl'])

    return pl;

def marginWithheld(openPositions):
    """
    Calculates the margin withheld in the open positions.

    Parameters-
    openPositions: Array of open positions.
    """
    totalMargin = 0.0
    for position in openPositions:
        totalMargin += float(position['margin'])

    return totalMargin

def calculateProfit(positions):
    """
    Calculates the total profit/loss from input positions.

    Parameters-
    positions: Array of positions.
    """
    totalProfit = 0.0
    for position in positions:
        totalProfit += float(position['pl'])

    return totalProfit

def realizedProfit(token):
    """
    Calculates the total profit/loss from closed positions.

    Parameters-
    token: Authentication token.
    """
    closedPositions = getPositions(token,"closed")['closed']
    return calculateProfit(closedPositions)

def unrealizedProfit(token):
    """
    Calculates the total profit/loss from open positions.

    Parameters-
    token: Authentication token.
    """
    openPositions = getPositions(token,"open")['open']
    return calculateProfit(openPositions)

def addMargin(token, pid, amount):
    """
    Adds margin to a running position.
 
    Parameters-
    token: Authentication token.
    pid: ID of the position.
    amount: amount to add in sats.
    """

    headers = {
        'content-type': "application/json",
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    payload = "{\"amount\":%d,\"pid\":\"%s\"}" % (amount,pid)
    positionData = requests.post(APIUrls.lnapi+APIUrls.addMarginUrl,data=payload,headers=headers)
    if positionData.status_code == 200:
        return positionData.json()
    else:
        raise RuntimeError('Unable to add margin to position %s, reason: %s' % (pid,positionData.text))

def cashin(token, pid, amount):
    """
    Retrieves part of a running position's profit.
 
    Parameters-
    token: Authentication token.
    pid: ID of the position.
    amount: amount to retrieve in sats.
    """

    headers = {
        'content-type': "application/json",
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    payload = "{\"amount\":%d,\"pid\":\"%s\"}" % (amount,pid)
    positionData = requests.post(APIUrls.lnapi+APIUrls.cashinnUrl,data=payload,headers=headers)
    if positionData.status_code == 200:
        return positionData.json()
    else:
        raise RuntimeError('Unable to cash-in position %s, reason: %s' % (pid,positionData.text))

def cancelPosition(token, pid):
    """
    Cancel the position linked to the given pid.Only works on positions that are not currently filled.
 
    Parameters-
    token: Authentication token.
    pid: ID of the position.
    """

    headers = {
        'content-type': "application/json",
        'accept': "application/json",
        'authorization': "Bearer %s" % token
    }
    payload = "{\"pid\":\"%s\"}" % pid
    positionData = requests.post(APIUrls.lnapi+APIUrls.cancelUrl,data=payload,headers=headers)
    if positionData.status_code == 200:
        return positionData.json()
    else:
        raise RuntimeError('Unable to cancel position %s, reason: %s' % (pid,positionData.text))
    
